@@ -135,7 +135,7 @@ function methodinstance_generator(world::UInt, source, self, ft::Type, tt::Type)
     # propagate edge metadata
     new_ci.min_world = min_world[]
     new_ci.max_world = max_world[]
-    new_ci.edges = MethodInstance[mi]
+    new_ci.edges = Any[mi]
 
     # prepare the slots
     new_ci.slotnames = Symbol[Symbol("#self#"), :ft, :tt]
@@ -501,31 +501,9 @@ CC.verbose_stmt_info(interp::GPUInterpreter) = false
 end
 CC.method_table(interp::GPUInterpreter) = interp.method_table_view
 
-# semi-concrete interepretation is broken with overlays (JuliaLang/julia#47349)
-function CC.concrete_eval_eligible(interp::GPUInterpreter,
-    @nospecialize(f), result::CC.MethodCallResult, arginfo::CC.ArgInfo, sv::CC.InferenceState)
-    # NOTE it's fine to skip overloading with `sv::IRInterpretationState` since we disables
-    #      semi-concrete interpretation anyway.
-    ret = @invoke CC.concrete_eval_eligible(interp::CC.AbstractInterpreter,
-        f::Any, result::CC.MethodCallResult, arginfo::CC.ArgInfo, sv::CC.InferenceState)
-    if ret === :semi_concrete_eval
-        return :none
-    end
-    return ret
-end
-function CC.concrete_eval_eligible(interp::GPUInterpreter,
-    @nospecialize(f), result::CC.MethodCallResult, arginfo::CC.ArgInfo)
-    ret = @invoke CC.concrete_eval_eligible(interp::CC.AbstractInterpreter,
-        f::Any, result::CC.MethodCallResult, arginfo::CC.ArgInfo)
-    ret === false && return nothing
-    return ret
-end
-
 
 ## world view of the cache
-if !isdefined(CC, :WorldView)
-    using Core.Compiler: InternalCodeCache
-else
+@static if VERSION < v"1.14-"
     using Core.Compiler: WorldView
 end
 
@@ -648,18 +626,18 @@ function ci_cache_populate(interp, cache, mi, min_world, max_world)
     return codeinfos
 end
 
+@static if VERSION >= v"1.14-"
 function ci_cache_lookup(cache, mi, min_world, max_world)
-    @static if !isdefined(CC, :WorldView)
-        # Use the overlay cache if possible
-        globalcache = cache.globalcache
-        cacheworlds = globalcache.worlds
-        if cacheworlds.min_world == min_world && cacheworlds.min_world == max_world
-            return CC.get(cache, mi, nothing)
-        end
-        wvc = InternalCodeCache(globalcache.owner, min_world, max_world)
-    else
-        wvc = WorldView(cache, min_world, max_world)
-    end
+    # In Julia 1.14+, WorldView was replaced by InternalCodeCache with WorldRange
+    # cache is OverlayCodeCache{InternalCodeCache}, extract owner from globalcache
+    owner = cache.globalcache.owner
+    wvc = CC.InternalCodeCache(owner, CC.WorldRange(min_world, max_world))
+    ci = CC.get(wvc, mi, nothing)
+    return ci
+end
+else
+function ci_cache_lookup(cache, mi, min_world, max_world)
+    wvc = WorldView(cache, min_world, max_world)
     ci = CC.get(wvc, mi, nothing)
     if VERSION < v"1.12.0-DEV.1434" && ci !== nothing && ci.inferred === nothing
         # if for some reason we did end up with a codeinfo without inferred source, e.g.,
@@ -669,6 +647,7 @@ function ci_cache_lookup(cache, mi, min_world, max_world)
     end
     return ci
 end
+end # @static if
 
 
 ## interface
@@ -704,6 +683,15 @@ function Base.precompile(@nospecialize(job::CompilerJob))
     return true
 end
 
+
+const HAS_LLVM_GET_CIS = (
+    VERSION >= v"1.13.0-DEV.1120" || (
+        Libdl.dlsym(
+            unsafe_load(cglobal(:jl_libjulia_handle, Ptr{Cvoid})), :jl_get_llvm_cis, throw_error = false
+        ) !== nothing
+    )
+)
+
 function compile_method_instance(@nospecialize(job::CompilerJob))
     if job.source.def.primary_world > job.world
         error("Cannot compile $(job.source) for world $(job.world); method is only valid from world $(job.source.def.primary_world) onwards")
@@ -800,27 +788,96 @@ function compile_method_instance(@nospecialize(job::CompilerJob))
         cache_gbl = nothing
     end
 
-    if VERSION >= v"1.13.0-DEV.623"
-        # Since Julia 1.13, the caller is responsible for initializing global variables that
-        # point to global values or bindings with their address in memory.
+    # Since Julia 1.13, the caller is responsible for initializing global variables that
+    # point to global values or bindings with their address in memory.
+    # Similarly on previous versions when imaging=true, it is also the caller's responsibility
+    # (see https://github.com/JuliaGPU/GPUCompiler.jl/issues/753), but we can support this on versions
+    # that have HAS_LLVM_GVS_GLOBALS.
+    gvs = nothing
+    inits = nothing
+    @static if VERSION >= v"1.13.0-DEV.623"
         num_gvars = Ref{Csize_t}(0)
         @ccall jl_get_llvm_gvs(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
-                               C_NULL::Ptr{Cvoid})::Nothing
+            C_NULL::Ptr{Cvoid}
+        )::Nothing
         gvs = Vector{Ptr{LLVM.API.LLVMOpaqueValue}}(undef, num_gvars[])
         @ccall jl_get_llvm_gvs(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
-                               gvs::Ptr{LLVM.API.LLVMOpaqueValue})::Nothing
+            gvs::Ptr{LLVM.API.LLVMOpaqueValue}
+        )::Nothing
+
         inits = Vector{Ptr{Cvoid}}(undef, num_gvars[])
         @ccall jl_get_llvm_gv_inits(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
                                     inits::Ptr{Cvoid})::Nothing
-
-        for (gv_ref, init) in zip(gvs, inits)
-            gv = GlobalVariable(gv_ref)
-            val = const_inttoptr(ConstantInt(Int64(init)), LLVM.PointerType())
-            initializer!(gv, val)
+    elseif HAS_LLVM_GVS_GLOBALS
+        if VERSION >= v"1.12.0-DEV.1703"
+            num_gvars = Ref{Csize_t}(0)
+            @ccall jl_get_llvm_gvs(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
+                C_NULL::Ptr{Cvoid}
+            )::Nothing
+            gvs = Vector{Ptr{LLVM.API.LLVMOpaqueValue}}(undef, num_gvars[])
+            @ccall jl_get_llvm_gvs_globals(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
+                gvs::Ptr{LLVM.API.LLVMOpaqueValue}
+            )::Nothing
+            inits = Vector{Ptr{Cvoid}}(undef, num_gvars[])
+            @ccall jl_get_llvm_gvs(native_code::Ptr{Cvoid}, num_gvars::Ptr{Csize_t},
+                inits::Ptr{Cvoid}
+            )::Nothing
+        else
+            gvs = get_llvm_global_vars(native_code)
+            inits = get_llvm_global_inits(native_code)
         end
     end
 
-    if VERSION >= v"1.13.0-DEV.1120"
+    # Maintain a map from global variables to their initialized Julia values.
+    # The objects pointed to are perma-rooted, during codegen.
+    # It is legal to call `Base.unsafe_pointer_to_objref` on `values(gv_to_value)`,
+    # but x->pointer_from_objref(Base.unsafe_pointer_to_objref(x)) is not idempotent,
+    # thus we store raw pointers here.
+    # Currently GVs are privatized, so users may have to handle embedded pointers,
+    # but this dictionary provides a clear indication that the embedded pointer is
+    # indeed avalid Julia object.
+    gv_to_value = Dict{String, Ptr{Cvoid}}()
+
+    # On certain version of Julia we have no reliable way to match the `gvs` to their initializers `inits`.
+    if gvs === nothing
+        # global variables here properly.
+        for gv in globals(llvm_mod)
+            if !haskey(metadata(gv), "julia.constgv")
+                continue
+            end
+            gv_to_value[LLVM.name(gv)] = C_NULL
+            val = initializer(gv)
+            if val === nothing
+                continue
+            end
+            while isa(val, LLVM.ConstantExpr)
+                if in(opcode(val), (LLVM.API.LLVMBitCast, LLVM.API.LLVMPtrToInt, LLVM.API.LLVMAddrSpaceCast, LLVM.API.LLVMIntToPtr))
+                    val = operands(val)[1]
+                    continue
+                end
+                break
+            end
+            if isa(val, LLVM.ConstantInt)
+                gv_to_value[LLVM.name(gv)] = reinterpret(Ptr{Cvoid}, convert(UInt, val))
+            end
+        end
+    else
+        @assert inits !== nothing
+        for (gv_ref, init) in zip(gvs, inits)
+            gv = GlobalVariable(gv_ref)
+            gv_to_value[LLVM.name(gv)] = init
+            # set the initializer
+            # TODO(vc): To enable full relocation we should actually strip out the initializers here.
+            if LLVM.isnull(initializer(gv))
+                val = const_inttoptr(ConstantInt(Int64(init)), value_type(initializer(gv)))
+                initializer!(gv, val)
+            end
+        end
+    end
+
+    code_instances = Core.CodeInstance[]
+
+    if HAS_LLVM_GET_CIS
         # on sufficiently recent versions of Julia, we can query the CIs compiled.
         # this is required after the move to `invoke(::CodeInstance)`, because our
         # lookup function (used to populate method_instances) isn't always called then.
@@ -828,14 +885,10 @@ function compile_method_instance(@nospecialize(job::CompilerJob))
         num_cis = Ref{Csize_t}(0)
         @ccall jl_get_llvm_cis(native_code::Ptr{Cvoid}, num_cis::Ptr{Csize_t},
                                C_NULL::Ptr{Cvoid})::Nothing
-        resize!(method_instances, num_cis[])
+        resize!(code_instances, num_cis[])
         @ccall jl_get_llvm_cis(native_code::Ptr{Cvoid}, num_cis::Ptr{Csize_t},
-                               method_instances::Ptr{Cvoid})::Nothing
-
-        for (i, ci) in enumerate(method_instances)
-            method_instances[i] = ci.def::MethodInstance
-        end
-
+            code_instances::Ptr{Cvoid}
+        )::Nothing
     elseif VERSION >= v"1.12.0-DEV.1703"
         # slightly older versions of Julia used MIs directly
 
@@ -847,11 +900,53 @@ function compile_method_instance(@nospecialize(job::CompilerJob))
                                method_instances::Ptr{Cvoid})::Nothing
     end
 
+    if !HAS_LLVM_GET_CIS
+        for mi in method_instances
+            ci = ci_cache_lookup(cache, mi, job.world, job.world)
+            ci === nothing && continue
+
+            llvm_func_idx = Ref{Int32}(-1)
+            llvm_specfunc_idx = Ref{Int32}(-1)
+            ccall(
+                :jl_get_function_id, Nothing,
+                (Ptr{Cvoid}, Any, Ptr{Int32}, Ptr{Int32}),
+                native_code, ci, llvm_func_idx, llvm_specfunc_idx
+            )
+            # Suppose we have two nested interpreters in use at the same time.
+            # Looking up a ci from the cache is not unique for a given mi.
+            # Consequently its possible we may not have compiled the ci found
+            # by the cache (instead having compiled the ci from the other interp).
+            if llvm_func_idx[] == -1
+                continue
+            end
+            push!(code_instances, ci)
+        end
+    else
+        # To avoid a clash in the compiled cache containing both with an interpreter token (like GPUCompiler.GPUCompilerCacheToken) and native,
+        # prefer the non-native code-instance.
+        # TODO: in the future we should migrate compiled to have the ci as the key, not the mi.
+        native_mis = Set{MethodInstance}()
+        for ci in code_instances
+            if ci.owner !== nothing
+                push!(native_mis, ci.def::MethodInstance)
+            end
+        end
+        filter!(code_instances) do ci
+            return ci.owner !== nothing || in(ci.def, native_mis)
+        end
+    end
+
+    # Avoid redundant code_instances. This is necessary to avoid false positives trying to add the same key'd mi to the compiled Dict.
+    unique!(code_instances)
+
+    resize!(method_instances, length(code_instances))
+    for (i, ci) in enumerate(code_instances)
+        method_instances[i] = ci.def::MethodInstance
+    end
+
     # process all compiled method instances
     compiled = Dict()
-    for mi in method_instances
-        ci = ci_cache_lookup(cache, mi, job.world, job.world)
-        ci === nothing && continue
+    for (ci, mi) in zip(code_instances, method_instances)
 
         # get the function index
         llvm_func_idx = Ref{Int32}(-1)
@@ -880,6 +975,8 @@ function compile_method_instance(@nospecialize(job::CompilerJob))
             nothing
         end
 
+        @assert !haskey(compiled, mi)
+
         # NOTE: it's not safe to store raw LLVM functions here, since those may get
         #       removed or renamed during optimization, so we store their name instead.
         compiled[mi] = (; ci, func=llvm_func, specfunc=llvm_specfunc)
@@ -888,7 +985,7 @@ function compile_method_instance(@nospecialize(job::CompilerJob))
     # ensure that the requested method instance was compiled
     @assert haskey(compiled, job.source)
 
-    return llvm_mod, compiled
+    return llvm_mod, compiled, gv_to_value
 end
 
 # partially revert JuliaLangjulia#49391
